/**
 * 
 * @author Micha Pawluczuk
 *
 *	RPC client
 *
 *	One call = one TCP connection
 *
 *	TODO
 *	- przerzuci obowizek sprawdzania nazwy metody w odpowiedzi na Peer albo TransportClient, do rozkminy
 *	- metoda leaves(predecessor)
 */
package budrys.chord;

public class Peer{
	protected ID peerID;
	protected IfcTransportClient tc;
	protected String address;
	protected Short port;
	
	/**
	 * This constructor does not connect to peer
	 * 
	 * @param id
	 * @param tc
	 */
	public Peer(Long id, IfcTransportClient tc){
		peerID = new ID(id);
		this.tc = (IfcTransportClient) tc.cclone();
	}
	
	/**
	 * This constructor connects to peer. To be used on bootstrap
	 * @param address
	 * @param port
	 * @param tc
	 */
	public Peer(String address, Short port, final IfcCallback callback, IfcTransportClient tc){
		this.tc = tc;
		this.address = address;
		this.port = port;
		
		// getId
		tc.request(this.address, this.port, "getId", "", new IfcCallback() {
			
			public void success(String command, String argument, Object cb) {
				if (command.equals("getId")){
					peerID = new ID(Long.valueOf((String)argument));
					callback.success(command, argument, null);
				} else {
					// response comes from other method, programmer error or hacking attempt
					callback.failure(new ChordException("Peer wrong response"), command);
				}
			}
			
			public void failure(Exception e, Object reason) {
				callback.failure(new ChordException("Failed to connect"), Peer.this.address+":"+Peer.this.port);
			}
		});
	}

	/**
	 * LOCAL
	 * @return
	 */
	public Long getId() {
		if (peerID == null){
			return null;
		}
		return peerID.getID();
	}
	
	/**
	 * Async
	 * @param address
	 * @param port
	 * @param callback
	 */
	public void getId(String address, Short port, IfcCallback callback) {
		tc.request(address, port, "getId", "", callback);
	}
	
	public Long getIdSync(String address, Short port) {
		Long result = Long.valueOf( tc.requestSync(address, port, "getId", "") );
		return result;
	}

	/**
	 * This method should be implemented iterative here
	 * @param id
	 * @param callerId 
	 * @param callback
	 */
	public void findSuccessor(final Long id, final Long callerId, final IfcCallback finalCallback) {
		final Long lastCalledPeerID = peerID.getID();
		tc.request(peerID.getIP(), peerID.getPort(), "findSuccessor", id.toString(), new IfcCallback() {
			
			public void success(String command, String argument, Object callback) {
				if (command.equals("findSuccessor")){
					finalCallback.success(command, argument, null);
				} else {
					if (command.equals("findSuccessorNotFound")){
						try {
							Thread.sleep(500);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						final Long bestIdSoFar = Long.valueOf(argument);
						Peer delegate = new Peer(bestIdSoFar, (IfcTransportClient) tc.cclone());
						//if (peerID.getID().equals(callerId)){
						if (bestIdSoFar.equals(callerId /*peerID.getID()*/)){
							// jesli ten kto przed chwila wywolal findSuccessor jest mn to jestem w petli nieskonczonej i jako finger wskaze sam siebie
							finalCallback.success("findSuccessor", peerID.getID().toString(), null);
						} else {
							// forward query around the circle
							//delegate.findSuccessor(id, peerID.getID() /*callerId*/, finalCallback);
							delegate.findSuccessor(id, peerID.getID() /*callerId*/, new IfcCallback(){
								
								public void success(String command, String argument, Object callback) {
									finalCallback.success(command, argument, null);
								}
								
								public void failure(Exception e, Object reason) {
									final Long justCalledPeerID = bestIdSoFar;
									finalCallback.failure(e, justCalledPeerID);
								}
								
							});
						}
						//delegate.findSuccessor(id, callerId, finalCallback);
					} else {
						// there are no other nodes in fingerlist, assuming we are the only node in the network
						finalCallback.success(command, peerID.getID().toString(), null);
					}
				}
				//callback
			}
			
			public void failure(Exception e, Object reason) {
				// peer has failed, caller needs to remove him from fingerlist asap
				finalCallback.failure(e, lastCalledPeerID);
			}
		});
		return ;
	}

	public void closestPrecedingNode(Long id, IfcCallback callback) {
		tc.request(peerID.getIP(), peerID.getPort(), "closestPrecedingNode", id.toString(), callback);
		return ;
	}
	
	public Long closestPrecedingNodeSync(Long id) {
		String r = tc.requestSync(peerID.getIP(), peerID.getPort(), "closestPrecedingNode", id.toString());
		Long result = null;
		if (r != null){
			result = Long.valueOf( r );
		}
		return result;
	}

	public void cNotify(Long id, IfcCallback callback) {
		tc.request(peerID.getIP(), peerID.getPort(), "cNotify", id.toString(), callback);
		return ;
	}
	
	public void ping(IfcCallback callback) {
		tc.request(peerID.getIP(), peerID.getPort(), "ping", "", callback);
		return ;
	}
	
	/**
	 * Tell me your predecessor
	 * @param id
	 * @param callback
	 */
	public void getPredecessor(IfcCallback callback) {
		tc.request(peerID.getIP(), peerID.getPort(), "getPredecessor", "", callback);
		return ;
	}
	
	public Long getPredecessorSync() {
		String r = tc.requestSync(peerID.getIP(), peerID.getPort(), "getPredecessor", "");
		Long result = null;
		Log.i("getPredSync:"+r);
		if (r != null){
			result = Long.valueOf( r );
		} else {
			return null;
		}
		
		return result;
	}
	
	public ID getPeerIdOffline(){
		return this.peerID;
	}

}
