package budrys.security;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import biz.source_code.base64Coder.Base64Coder;

public class Crypt {
//	private static final String ALGORITHM = "RSA/ECB/OAEPWithSHA1AndMGF1Padding";
	private static final String RSA_ALGORITHM = "RSA";
	public static KeyPair generateKeys(){
		KeyPairGenerator kpg;
		KeyPair kp = null;
		try {
			kpg = KeyPairGenerator.getInstance("RSA");
			kpg.initialize(2048);
	        kp = kpg.genKeyPair();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        return kp;
	}
	
	/**
	 * 
	 * @param pub
	 * @return base64 encoded key
	 */
	public static String encodeKey(RSAPublicKey pub){
		return new String(Base64Coder.encode(pub.getEncoded()));
	}
	
	public static String encodeKey(RSAPrivateKey priv){
		return new String(Base64Coder.encode(priv.getEncoded()));
	}
	
	public static RSAPublicKey decodePublicKey(String base64pub){
		byte[] base64decoded = Base64Coder.decode(base64pub);
		X509EncodedKeySpec ks = new X509EncodedKeySpec(base64decoded);
        KeyFactory kf;
        RSAPublicKey pubKey2 = null;
		try {
			kf = KeyFactory.getInstance("RSA");
			pubKey2 = (RSAPublicKey) kf.generatePublic(ks);
		} catch (InvalidKeySpecException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e){
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        return pubKey2;
	}
	
	public static RSAPrivateKey decodePrivateKey(String base64priv){
		byte[] base64decoded = Base64Coder.decode(base64priv);
		PKCS8EncodedKeySpec privateKeySpec = new PKCS8EncodedKeySpec(base64decoded);
        KeyFactory kf2;
        RSAPrivateKey privKey2 = null;
		try {
			kf2 = KeyFactory.getInstance("RSA");
			privKey2 = (RSAPrivateKey) kf2.generatePrivate(privateKeySpec);
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidKeySpecException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        return privKey2;
	}
	
	/**
	 * 
	 * @param msg original message
	 * @param priv RSA private key
	 * @return base64 encoded signature
	 */
	public static String signMessage(String msg, RSAPrivateKey priv){
		byte[] cipherText = new byte[0];
		try {
			Cipher cipher = Cipher.getInstance(RSA_ALGORITHM);
			cipher.init(Cipher.ENCRYPT_MODE, priv);
			byte[] hashedMsg = secureHash(msg);
			cipherText = cipher.doFinal(hashedMsg);
		} catch (NoSuchPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e){
			// TODO Auto-generated catch block
						e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BadPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return new String(Base64Coder.encode(cipherText));
	}
	
	/**
	 * 
	 * @param msg original message
	 * @param signature  base64 encoded signature
	 * @param pub RSA public key
	 * @return
	 */
	public static boolean verifySignature(String msg, String signature, RSAPublicKey pub){
		byte[] cipherText = new byte[0];
		try {
			Cipher cipher = Cipher.getInstance(RSA_ALGORITHM);
			cipher.init(Cipher.DECRYPT_MODE, pub);
			byte[] calculatedHash = secureHash(msg);
			String calculatedHashString = new String(Base64Coder.encode(calculatedHash));
			byte[] decodedSignature = Base64Coder.decode(signature);
			cipherText = cipher.doFinal(decodedSignature);
			String decryptedSignatureHashString = new String(Base64Coder.encode(cipherText));
//			System.out.println("decryptedSignatureHash: "+decryptedSignatureHashString);
			
			if (calculatedHashString.equals(decryptedSignatureHashString)){
				// verified OK
				return true;
			} else {
				// not verified
				return false;
			}
			
		} catch (NoSuchPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BadPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}
	
	private static byte[] secureHash(String msg){
		byte[] result = new byte[0];
		try {
			MessageDigest md;
			md = MessageDigest.getInstance("MD5");
			md.reset();
			md.update(msg.getBytes("UTF-8"));
			result = md.digest();
//			String calculatedHash = new String(Base64Coder.encode(result));
//			System.out.println("calculatedHash: "+calculatedHash);
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return result;
	}
	
	private static String secureHashEncoded(String msg){
		return new String(Base64Coder.encode(secureHash(msg)));
	}
	
	/**
	 * 
	 * @param msg
	 * @param pub
	 * @return base64 encoded encrypted message
	 */
	public static String encryptMessage(String msg, RSAPublicKey pub){
		byte[] cipherText = new byte[0];
		try {
			Cipher cipher = Cipher.getInstance(RSA_ALGORITHM);
			cipher.init(Cipher.ENCRYPT_MODE, pub);
			byte[] byteMsg = msg.getBytes("UTF-8");
			cipherText = cipher.doFinal(byteMsg);
		} catch (NoSuchPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e){
			// TODO Auto-generated catch block
						e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BadPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return new String(Base64Coder.encode(cipherText));
	}
	
	/**
	 * 
	 * @param encryptedMsg
	 * @param pub
	 * @return null if incorrect or error
	 */
	public static String decryptMessage(String encryptedMsg, RSAPrivateKey priv){
		String decryptedMsg = null;
		try {
			Cipher cipher = Cipher.getInstance(RSA_ALGORITHM);
			cipher.init(Cipher.DECRYPT_MODE, priv);
			byte[] decryptedBytes = Base64Coder.decode(encryptedMsg);
			byte[] cipherText = cipher.doFinal(decryptedBytes);
			decryptedMsg = new String(cipherText, "UTF-8");
		} catch (NoSuchPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BadPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return decryptedMsg;
	}
	
}
