package budrys.social;

import java.security.KeyPair;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jsonb.JSONArray;
import org.jsonb.JSONException;
import org.jsonb.JSONObject;

import budrys.security.Crypt;

public class ProfileModel{
	// social public
	private RSAPublicKey publicKey;
	private String signature;
	private String name, surname;
	private String address;
	private List<ProfileWallEntry> wall;
	// social restricted (after authorization)
	private String restricted; 

	// social private
	private RSAPrivateKey privateKey;
	// not social
	private String avatarPath = null;
	private boolean signatureVerified = false;
	private boolean fromCache = false;

	public ProfileModel(String nName, String nSurname, String nAddress ) {
		this.name = nName;
		this.surname = nSurname;
		this.address = nAddress;
		wall = new ArrayList<ProfileWallEntry>();
		wall.add(new ProfileWallEntry("Hello", "10-12-2012 10:20", "Rick Roll"));
		wall.add(new ProfileWallEntry("Sprzedam Opla !", "15-12-2012 12:50", "Rick Roll"));
	}
	
	public ProfileModel(String nName, String nSurname, String nAddress, boolean owner ) {
		this.name = nName;
		this.surname = nSurname;
		this.address = nAddress;
		wall = new ArrayList<ProfileWallEntry>();
		wall.add(new ProfileWallEntry("Hello", "10-12-2012 10:20", "Rick Roll"));
		wall.add(new ProfileWallEntry("Sprzedam Opla !", "15-12-2012 12:50", "Rick Roll"));
		if (owner){
			// generate keys if null
			if ((this.publicKey == null) || (this.privateKey == null)){
				this.generateKeys();
			}
		}
	}
	
	public ProfileModel(String json){
		wall = new ArrayList<ProfileWallEntry>();
		importFromJson(json);
	}
	
	public ProfileModel(String json, boolean owner){
		wall = new ArrayList<ProfileWallEntry>();
		importFromJson(json);
		if (owner){
			// generate keys if null
			if ((this.publicKey == null) || (this.privateKey == null)){
				this.generateKeys();
			}
		}
	}
	
	private void generateKeys(){
		KeyPair kp = Crypt.generateKeys();
		this.privateKey = (RSAPrivateKey) kp.getPrivate();
		this.publicKey = (RSAPublicKey) kp.getPublic();
	}
	
	/**
	 * Imports profile data from JSON, might be just public data or all data
	 * @param json
	 * @return
	 */
	public boolean importFromJson(String json2){
		JSONObject decode;
		JSONObject decode2;
		try {
			signatureVerified = false;
			decode2 = new JSONObject(json2.toString());
			decode = decode2.getJSONObject("data");
			this.name = decode.getString("name");
			this.surname = decode.getString("surname");
			this.address = decode.getString("address");

			// KEYS
			if (decode.has("publicKey")){
				String publicKeyString = decode.getString("publicKey");
				this.publicKey = Crypt.decodePublicKey(publicKeyString);
				// SIGNATURE
				if (decode2.has("signature")){
					String signedData = json2.substring(8, decode.toString().length()+8);		
					this.signature = decode2.getString("signature");
					this.signatureVerified = Crypt.verifySignature(signedData, this.signature, this.publicKey);
				}
			}
			if (decode.has("privateKey")){
				String privateKeyString = decode.getString("privateKey");
				this.privateKey = Crypt.decodePrivateKey(privateKeyString);
			}
			
			
			// WALL
			wall = new ArrayList<ProfileWallEntry>();
			if (decode.has("wall")){
				JSONArray jb = decode.getJSONArray("wall");
				for (int i=0; i<jb.length(); ++i){
					JSONObject jo = jb.getJSONObject(i);
					ProfileWallEntry pwe = new ProfileWallEntry(jo.getString("message"), jo.getString("date"), jo.getString("name"));
					wall.add(pwe);
				}
			}
			
			// RESTRICTED
			if (decode.has("restricted")){
				this.restricted = decode.getString("restricted");				
			}
			if (decode.has("avatarPath")){
				this.avatarPath = decode.getString("avatarPath");				
			}
			
			
		} catch (JSONException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	/**
	 * exports ALL profile data to json
	 * if you want to export just public data then use exportPublicToJson or exportRestrictedToJson
	 * public < restricted < all (private)
	 * @return
	 */
	public JSONObject exportAllToJson(){
		JSONObject json = new JSONObject();
		JSONObject json2 = new JSONObject();
		try {
			json.accumulate("name", name);
			json.accumulate("surname", surname);
			json.accumulate("address", address);
			Iterator<ProfileWallEntry> it = wall.iterator();
			while (it.hasNext()){
				ProfileWallEntry pwe = it.next();
				JSONObject wallEntry = new JSONObject();
				wallEntry.put("date", pwe.getDate());
				wallEntry.put("message", pwe.getMessage());
				wallEntry.put("name", pwe.getName());
				json.append("wall", wallEntry);
			}
			// PUBLIC
			if (this.publicKey != null){
				json.accumulate("publicKey", Crypt.encodeKey(this.publicKey) );
			}
			// RESTRICTED
			json.accumulate("restricted", this.restricted);
			// PRIVATE
			if (this.privateKey != null){
				json.accumulate("privateKey", Crypt.encodeKey(this.privateKey) );
			}
			if (this.avatarPath != null){
				json.accumulate("avatarPath", avatarPath);
			}
			// SIGNATURE
			if (this.privateKey != null){
				this.signature = Crypt.signMessage(json.toString(), privateKey);
			} else {
//				this.signature = null;
			}
			json2.accumulate("data", json);
			json2.accumulate("signature", this.signature);
			
		} catch (JSONException e1) {
			e1.printStackTrace();
		}
		return json2;
	}

	public JSONObject exportPublicToJson(){
		JSONObject json = new JSONObject();
		JSONObject json2 = new JSONObject();
		try {
			// PUBLIC
			if (this.publicKey != null){
				json.accumulate("publicKey", Crypt.encodeKey(this.publicKey) );
			}
			json.accumulate("name", name);
			json.accumulate("surname", surname);
			json.accumulate("address", address);
			Iterator<ProfileWallEntry> it = wall.iterator();
			while (it.hasNext()){
				ProfileWallEntry pwe = it.next();
				JSONObject wallEntry = new JSONObject();
				wallEntry.put("date", pwe.getDate());
				wallEntry.put("message", pwe.getMessage());
				wallEntry.put("name", pwe.getName());
				json.append("wall", wallEntry);
			}
			// SIGNATURE
			if (this.privateKey != null){
				this.signature = Crypt.signMessage(json.toString(), privateKey);
			} else {
//				this.signature = null;
			}
			json2.accumulate("data", json);
			json2.accumulate("signature", this.signature);
		} catch (JSONException e1) {
			e1.printStackTrace();
		}
		return json2;
	}

	public JSONObject exportRestrictedToJson(){
		JSONObject json = new JSONObject();
		JSONObject json2 = new JSONObject();
		try {
			// PUBLIC
			if (this.publicKey != null){
				json.accumulate("publicKey", Crypt.encodeKey(this.publicKey) );
			}
			json.accumulate("name", name);
			json.accumulate("surname", surname);
			json.accumulate("address", address);
			Iterator<ProfileWallEntry> it = wall.iterator();
			while (it.hasNext()){
				ProfileWallEntry pwe = it.next();
				JSONObject wallEntry = new JSONObject();
				wallEntry.put("date", pwe.getDate());
				wallEntry.put("message", pwe.getMessage());
				wallEntry.put("name", pwe.getName());
				json.append("wall", wallEntry);
			}
			// RESTRICTED
			json.accumulate("restricted", this.restricted);
			// SIGNATURE
			if (this.privateKey != null){
				this.signature = Crypt.signMessage(json.toString(), privateKey);
			} else {
//				this.signature = null;
			}
			json2.accumulate("data", json);
			json2.accumulate("signature", this.signature);
		} catch (JSONException e1) {
			e1.printStackTrace();
		}
		return json2;
	}
	
	public String getAvatarPath() {
		return avatarPath;
	}

	public void setAvatarPath(String avatarPath) {
		this.avatarPath = avatarPath;
	}
	
	public boolean isFromCache() {
		return fromCache;
	}

	public void setFromCache(boolean fromCache) {
		this.fromCache = fromCache;
	}
	
	public String getSignature() {
		return signature;
	}

	public boolean isSignatureVerified() {
		return signatureVerified;
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}
	
	public RSAPublicKey getPublicKey() {
		return this.publicKey;
	}
	
	public void removeFromWall(int index){
		wall.remove(wall.size() - index - 1);
	}
	
	public List<ProfileWallEntry> getWall() {
		return wall;
	}
	
	public String getRestricted() {
		return restricted;
	}

	public void setRestricted(String restricted) {
		this.restricted = restricted;
	}
	
//	public String decryptMessage(String crypted){
//		return Crypt.decryptMessage(crypted, this.privateKey);
//	}
}
