package budrys.social;

import java.security.interfaces.RSAPublicKey;
import budrys.chord.IfcCallback;
import budrys.chord.IfcTransportServer;
import budrys.security.Crypt;

public class ProfileService {
	private IfcTransportServer ts;	// main ts
	private Short myPort;

	private ProfileModel pmodel;
//	private FriendsModel friends;
	
	public ProfileService(Short myPort, IfcTransportServer ts, ProfileModel nModel, final FriendsModel friends, final IfcCallback onReceivePubMsg, final IfcCallback onFileRequest) {
		this.myPort = myPort;
		this.ts = ts;
		this.pmodel = nModel;
//		this.friends = friends;
		
		this.ts.startService(this.myPort, new IfcCallback(){
			
			public void success(String command, String argument, final Object callback) { // on RPC receive request
				IfcCallback cb = (IfcCallback)callback;
				
				// PROCESS COMMANDS
				if (command.equals("getProfileData")){
					// if private signed message attached then verify and if verified then send restricted data instead of public
					if ((argument!=null)&&(argument!="")&&(friends.isFriend(argument))){
//						RSAPublicKey pubKey = Crypt.decodePublicKey(argument);
//						String encryptedProfileData = Crypt.encryptMessage(ProfileService.this.pmodel.exportRestrictedToJson().toString(), pubKey);
//						cb.success("getProfileDataEncrypted", encryptedProfileData, null);
						cb.success("getProfileData", ProfileService.this.pmodel.exportRestrictedToJson().toString(), null);
					} else {
						cb.success("getProfileData", ProfileService.this.pmodel.exportPublicToJson().toString(), null);						
					}
				} else if (command.equals("publishWallMessage")){
					if ((argument!=null)&&(argument!="")){
						ProfileWallEntry pwe = new ProfileWallEntry(argument);
						ProfileService.this.pmodel.getWall().add(pwe);
						onReceivePubMsg.success(command, pwe.exportToJson().toString(), null);
						cb.success("publishWallMessage", "ok", null);
						// TODO
						// if signature not valid then send success with message sigfail
					}
				} else if (command.equals("getAvatar")){
					// BINARY RESPONSE
					onFileRequest.success("avatar", null, new IfcCallback() {
						public void success(String command, String argument, Object binary) {
							byte[] binaryData = (byte[])binary;
							((IfcCallback) callback).success(null, null, binaryData);
						}
						public void failure(Exception e, Object reason) {
							// file not found
							((IfcCallback) callback).success(null, null,  new byte[1]);
						}
					});
//					ProfileService.this.pmodel.getAvatarPath()
//					cb.success("publishWallMessage", "ok", null);
				} else {
					cb.failure(new Exception(), "No such command: " + command);
					clog("No such command: " + command + "("+argument+")");
				}
				
			}
			
			public void failure(Exception e, Object reason) {
				clog("Error processing profile RPC: " + e.toString());
			}
		});
	}
	
	public void stopService(){
		this.ts.stopService();
	}
	
	public Short getMyPort() {
		return myPort;
	}
	
	public ProfileModel getProfileModel(){
		return pmodel;
	}
	
	private void setProfileModel(ProfileModel pm){
		this.pmodel = pm;
	}
	
	private synchronized void clog(String s){
		System.out.println(myPort + " " + s);
	}
	
	
}
