/**
 * 
 * @author Micha Pawluczuk
 *
 */
package budrys.transport.socket;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketTimeoutException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Iterator;

import budrys.chord.IfcCallback;
import budrys.chord.IfcTransportClient;
import budrys.chord.Log;

public class TransportClient implements IfcTransportClient {
	
	private Boolean terminated = false;
	String result = null;
	
	
	public Object cclone() {
		TransportClient result = new TransportClient();
		return result;
	}
	

	
	/**
	 * NON BLOCKING - ASYNCHRONOUS
	 * TODO starting a new thread should be wrapping sync method, now it is done vice versa
	 */
	public void request(final String remoteIp, final Short remotePort, String command, String argument,
			final IfcCallback callback, final boolean binaryResponse) {
		final TransportPacket toSend = new TransportPacket(command, argument);
		
		(new Thread() {
			public void run() {
				try{
					SocketAddress sockaddr = new InetSocketAddress(remoteIp, remotePort);

					// Create an unbound socket
					Socket client = new Socket();

					// This method will block no more than timeoutMs.
					// If the timeout occurs, SocketTimeoutException is thrown.
					int timeoutMs = 5000; // 5 seconds
					client.connect(sockaddr, timeoutMs);
					
					ObjectOutputStream out;
					try{
						out = new ObjectOutputStream(client.getOutputStream());
						out.flush();
						
						try{
							out.writeObject(toSend);
							if (!binaryResponse){
								// RECEIVE text
								ObjectInputStream in;
								try{
									in = new ObjectInputStream(client.getInputStream());
									
									try{
										TransportPacket receivedPacket = (TransportPacket) in.readObject();
										callback.success(receivedPacket.command, receivedPacket.argument, null);
									}
									catch (IOException e){
										callback.failure(e, null);
									}
									catch (ClassNotFoundException e){
										callback.failure(e,null);
									}
									finally{
										// TODO close connection
										in.close();
										client.close();
									}
								}
								catch (IOException e1){
									// exception here means that client got disconnected
									try{
										client.close();
										callback.failure(e1, null);
									}
									catch (IOException e){
										callback.failure(e, null);
									}
									finally{
										client.close();
									}
								}
								// end receive text
							} else {
								// RECEIVE binary
								InputStream is = client.getInputStream();
								int count;
								ArrayList<byte[]> bytesList = new ArrayList<byte[]>();
								byte[] justReceived = new byte[1024];
							    while ((count = is.read(justReceived)) > 0) {
							    	bytesList.add(justReceived.clone());
							    }
							    byte[] wholeData = new byte[bytesList.size()*1024];
							    Iterator<byte[]> it = bytesList.iterator();
							    int destPos = 0;
							    while (it.hasNext()){
							    	byte[] c = it.next();
//							    	System.arraycopy(src, 	srcPos, dst, 		dstPos, 	length);
							    	System.arraycopy(c,		0, 		wholeData, 	destPos, 	c.length);
							    	destPos+=c.length;
							    }
							    callback.success("binary", null, wholeData);
							    is.close();
								// end receive binary
							}
							
							
						}
						catch (IOException e){
							callback.failure(e, null);
						}
						finally{
							out.close();
							client.close();
						}
					}
					catch (IOException e){
						try{
							client.close();
							callback.failure(e, null);
						}
						catch (IOException e1){
							callback.failure(e1, null);
						}
					}
					
				}
				catch (UnknownHostException e){
					callback.failure(e, null);
				}
				catch (SocketTimeoutException e){
					callback.failure(e, null);
				}
				catch (IOException e){
					callback.failure(e, null);
				}
				finally{
					//
				}
			}
		}).start();
		
		return ;
	}
	
	public void request(final String remoteIp, final Short remotePort, String command, String argument,
			final IfcCallback callback){
		request(remoteIp, remotePort, command, argument, callback, false);
	}





	/**
	 * Active waiting
	 */
	public String requestSync(String remoteIp, Short remotePort,
			String command, String argument) {
		result = null;
		terminated = false; 
		
		request(remoteIp, remotePort, command, argument, new IfcCallback() {
			
			public void success(String command, String argument, Object callback) {
				terminated = true;
				result = argument+"";
			}
			
			public void failure(Exception e, Object reason) {
				terminated = true;
				result = null;
			}
		});
		
		while(!terminated){	// ACTIVE WAITING
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			Log.i("tc.requestSync:waiting");
		}
		
		return result;
	}

}
